"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/example-greeting-app
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const context_1 = require("@loopback/context");
const rest_1 = require("@loopback/rest");
const debugFactory = require("debug");
const caching_service_1 = require("../caching-service");
const keys_1 = require("../keys");
const debug = debugFactory('greeter-extension');
let CachingInterceptor = class CachingInterceptor {
    constructor(cachingService) {
        this.cachingService = cachingService;
    }
    value() {
        return async (ctx, next) => {
            const httpReq = await ctx.get(rest_1.RestBindings.Http.REQUEST, {
                optional: true,
            });
            /* istanbul ignore if */
            if (!httpReq) {
                // Not http request
                return await next();
            }
            const key = httpReq.path;
            const lang = httpReq.acceptsLanguages(['en', 'zh']) || 'en';
            const cachingKey = `${lang}:${key}`;
            const cachedResult = await this.cachingService.get(cachingKey);
            if (cachedResult) {
                debug('Cache found for %s %j', cachingKey, cachedResult);
                return cachedResult;
            }
            const result = await next();
            await this.cachingService.set(cachingKey, result);
            return result;
        };
    }
};
CachingInterceptor = __decorate([
    context_1.bind(context_1.asGlobalInterceptor('caching')),
    __param(0, context_1.inject(keys_1.CACHING_SERVICE)),
    __metadata("design:paramtypes", [caching_service_1.CachingService])
], CachingInterceptor);
exports.CachingInterceptor = CachingInterceptor;
//# sourceMappingURL=caching.interceptor.js.map